@@ -4,15 +4,17 @@ use std::str::FromStr;
 
 use crate::draw_commands::DrawCommand;
 use crate::storage::Storage;
-use crate::shape::{Shape, ShapeType, ShapeFinished};
+use crate::shape::{Shape, ShapeType, ShapeFinished, ShapeBuilder};
 use crate::color::Color;
 use crate::action::Action;
-use crate::point::{Vec2DDrawing, Vec2DScreen};
+use crate::point::{Point, Vec2DWorld, Vec2DScreen};
 use crate::transform::Transform;
 use crate::consts::{DEFAULT_THICKNESS, TOUCH_RADIUS};
 use crate::deserialize;
+use crate::geom::{Angle, bbox_from_points};
+use crate::key::Key;
 
-#[derive(Copy,Clone,Debug)]
+#[derive(Copy,Clone,Debug, PartialEq, Eq)]
 /// Describes the possible states of the undo_queue and a pointer that moves
 /// in it
 enum UndoState {
@@ -30,7 +32,7 @@ enum UndoState {
 }
 
 /// Manages what is happening with the board
-#[derive(Copy, Clone)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 enum BoardState {
     Idle,
     UsingTool,
@@ -52,6 +54,7 @@ pub enum ShouldRedraw {
     No,
 }
 
+#[derive(Debug)]
 pub enum SelectedTool {
     Shape(ShapeType),
     Eraser,
@@ -69,7 +72,7 @@ impl FromStr for SelectedTool {
 }
 
 /// Possible statuses of file save
-#[derive(Clone)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub enum SaveStatus {
     NewAndEmpty,
     NewAndChanged,
@@ -78,6 +81,7 @@ pub enum SaveStatus {
 }
 
 /// Mouse input management
+#[derive(Debug)]
 pub enum MouseButton {
     Left,
     Middle,
@@ -85,8 +89,13 @@ pub enum MouseButton {
     Unknown,
 }
 
-pub struct App {
+/// Main controller of the pizarra application.
+///
+/// This struct keeps the state of a drawing and exposes the methods necessary
+/// to mutate it according to events in the interface.
+pub struct Pizarra {
     storage: Storage,
+    current_shape: Option<Box<dyn ShapeBuilder>>,
 
     selected_tool: SelectedTool,
     selected_color: Color,
@@ -95,9 +104,10 @@ pub struct App {
     alpha: f64,
     erase_radius: f64,
 
-    /// This value represents the coordinate of the drwing space that is currently
-    /// at the top-left corner of the screen.
+    /// The transformation matrix that takes points from the drawing coordinate
+    /// system to the screen coordinate system.
     transform: Transform,
+    /// What is the  size of the canvas in pixels?
     dimensions: Vec2DScreen,
 
     /// Stores erased shapes temporarily while being erased
@@ -109,17 +119,15 @@ pub struct App {
 
     /// What is happening with the board right now?
     board_state: BoardState,
+    /// Is the shift key pressed on the keyboard?
+    is_shift_pressed: bool,
 
     /// File management
     save_status: SaveStatus,
 }
 
 // Private methods
-impl App {
-    /// p is a point in the coordinate space of the drawing area. This function
-    /// converts p's coordinates to storage coordinates given the current zoom
-    /// and offset.
-
+impl Pizarra {
     fn conclude_action(&mut self, action: Action) {
         // this means some undos have been applied. Doing something from here
         // means effectively dropping some actions such that we're again in the
@@ -146,23 +154,23 @@ impl App {
     }
 
     fn handle_tool_button_pressed(&mut self, point: Vec2DScreen) -> ShouldRedraw {
-        let transformed_point = self.transform.to_storage_coordinates(point);
+        let transformed_point = self.transform.to_world_coordinates(point);
 
         match self.selected_tool {
             SelectedTool::Shape(shape) => {
-                if let Some(shape) = self.storage.last_mut() {
-                    shape.handle_button_pressed(transformed_point);
+                if let Some(shape) = self.current_shape.as_mut() {
+                    shape.handle_button_pressed(point, self.transform);
                 } else {
-                    let new_shape = shape.start(self.selected_color.with_alpha(self.alpha), transformed_point, self.stroke * 1.0 / self.transform.scale_factor());
+                    let new_shape = shape.start(self.selected_color.with_alpha(self.alpha), transformed_point, self.transform.to_world_units(self.stroke));
 
-                    self.storage.add(new_shape);
+                    self.current_shape = Some(new_shape);
                 }
 
                 ShouldRedraw::Shape
             },
             SelectedTool::Eraser => {
                 let deleted_shape = self.storage
-                    .query_circle(transformed_point, self.erase_radius / self.transform.scale_factor())
+                    .query_circle(transformed_point, self.transform.to_world_units(self.erase_radius))
                     .map(|id| self.storage.remove(id))
                     .flatten();
 
@@ -178,29 +186,29 @@ impl App {
     }
 
     fn handle_tool_button_released(&mut self, point: Vec2DScreen) -> Action {
-        let transformed_point = self.transform.to_storage_coordinates(point);
+        let transformed_point = self.transform.to_world_coordinates(point);
 
         match self.selected_tool {
             SelectedTool::Shape(_shape) => {
-                let finished = if let Some(shape) = self.storage.last_mut() {
-                    shape.handle_button_released(transformed_point)
+                let finished = if let Some(shape) = self.current_shape.as_mut() {
+                    shape.handle_button_released(point, self.transform)
                 } else {
                     ShapeFinished::No
                 };
 
-                if let ShapeFinished::Yes = finished {
-                    if let Some(shape_id) = self.storage.flush() {
-                        Action::Draw(shape_id)
-                    } else {
-                        Action::Empty
-                    }
+                if let ShapeFinished::Yes(shape) = finished {
+                    let shape_id = self.storage.add(shape);
+
+                    self.current_shape = None;
+
+                    Action::Draw(shape_id)
                 } else {
                     Action::Unfinished
                 }
             },
             SelectedTool::Eraser => {
                 let deleted_shape = self.storage
-                    .query_circle(transformed_point, self.erase_radius / self.transform.scale_factor())
+                    .query_circle(transformed_point, self.transform.to_world_units(self.erase_radius))
                     .map(|id| self.storage.remove(id))
                     .flatten();
 
@@ -218,12 +226,12 @@ impl App {
     }
 
     fn handle_tool_mouse_move(&mut self, point: Vec2DScreen) -> ShouldRedraw {
-        let transformed_point = self.transform.to_storage_coordinates(point);
+        let transformed_point = self.transform.to_world_coordinates(point);
 
         match self.selected_tool {
             SelectedTool::Shape(_shape_type) => {
-                if let Some(shape) = self.storage.last_mut() {
-                    shape.handle_mouse_moved(transformed_point);
+                if let Some(shape) = self.current_shape.as_mut() {
+                    shape.handle_mouse_moved(point, self.transform);
                     ShouldRedraw::Shape
                 } else {
                     ShouldRedraw::No
@@ -231,7 +239,7 @@ impl App {
             },
             SelectedTool::Eraser => {
                 let deleted_shape = self.storage
-                    .query_circle(transformed_point, self.erase_radius / self.transform.scale_factor())
+                    .query_circle(transformed_point, self.transform.to_world_units(self.erase_radius))
                     .map(|id| self.storage.remove(id))
                     .flatten();
 
@@ -261,8 +269,6 @@ impl App {
             Action::Erase(shapes) => {
                 let ids = shapes.into_iter().map(|shape| self.storage.restore(shape)).collect();
 
-                self.storage.flush();
-
                 Action::Redraw(ids)
             },
 
@@ -273,16 +279,61 @@ impl App {
             },
         }
     }
+
+    /// Move the canvas by this offset
+    fn translate(&mut self, delta: Vec2DScreen) {
+        self.transform = self.transform.r#move(delta);
+    }
+
+    fn rotate(&mut self, angle: Angle, fixed: Vec2DScreen) {
+        self.transform = self.transform.turn(angle, fixed);
+    }
+
+    /// Handles the canvas moving logic. Must be kept private
+    fn handle_move(&mut self, old_point: Vec2DScreen, new_point: Vec2DScreen) {
+        if self.is_shift_pressed {
+            let center = self.dimensions / 2.0;
+
+            let a = old_point - center;
+            let b = new_point - center;
+
+            self.rotate(Angle::between(a, b), center);
+        } else {
+            let delta = new_point - old_point;
+
+            self.translate(delta);
+        }
+    }
+
+    /// Compute the bounds of what is visible in the board according to current
+    /// offset, zoom level and rotation. Return value is an array of the
+    /// top-left, top-right, bottom-right, and bottom-left corners.
+    fn visible_extent(&self) -> [Vec2DWorld; 4] {
+        [
+            self.transform.to_world_coordinates(Vec2DScreen::new(0.0, 0.0)),
+            self.transform.to_world_coordinates(Vec2DScreen::new(self.dimensions.x, 0.0)),
+            self.transform.to_world_coordinates(self.dimensions),
+            self.transform.to_world_coordinates(Vec2DScreen::new(0.0, self.dimensions.y)),
+        ]
+    }
+
+    /// return the screen-axis-aligned bounding box of the visible portion of
+    /// the screen.
+    fn visible_bbox(&self) -> [Vec2DWorld; 2] {
+        bbox_from_points(self.visible_extent().iter().copied())
+    }
 }
 
 // Public interface
-impl App {
+impl Pizarra {
     /// Create a new instance of Pizarra's controller. Only one is needed and it
     /// can be put inside an Rc<RefCell<_>> to use it inside event callbacks in
     /// UI implementations.
-    pub fn new(dimensions: Vec2DScreen) -> App {
-        App {
+    pub fn new(dimensions: Vec2DScreen) -> Pizarra {
+        Pizarra {
             storage: Storage::new(),
+            current_shape: None,
+
             selected_tool: SelectedTool::Shape(ShapeType::Path),
             bgcolor: Color::black(),
             stroke: DEFAULT_THICKNESS,
@@ -291,12 +342,13 @@ impl App {
 
             /// a vector from the top-left corner of the screen to the origin
             /// of coordinates in the storage
-            transform: Transform::new((dimensions * -0.5).into(), 1.0),
+            transform: Transform::default_for_viewport(dimensions),
             dimensions,
 
             erased: Vec::new(),
 
             board_state: BoardState::Idle,
+            is_shift_pressed: false,
 
             undo_state: UndoState::Reset,
             undo_queue: Vec::new(),
@@ -313,15 +365,6 @@ impl App {
         self.transform
     }
 
-    /// Compute the bounds of what is visible in the board according to current
-    /// offset and zoom level
-    pub fn visible_extent(&self) -> [Vec2DDrawing; 2] {
-        [
-            self.transform.offset().into(),
-            (self.transform.offset() + (self.dimensions.to_vec2d() / self.transform.scale_factor())).into()
-        ]
-    }
-
     /// Returns the dimensions of the screen
     pub fn get_dimensions(&self) -> Vec2DScreen {
         self.dimensions
@@ -335,32 +378,31 @@ impl App {
     /// Returns an iterator over the draw commands required to render the visible
     /// portion of the drawing
     pub fn draw_commands_for_screen(&self) -> Vec<DrawCommand> {
-        self.storage.draw_commands(self.visible_extent())
+        self.storage.draw_commands(self.visible_bbox())
     }
 
     /// Returns an interator over the draw commands required to render the wole
-    /// drawing
+    /// drawing. Used for exporting to png
     pub fn draw_commands_for_drawing(&self) -> Vec<DrawCommand> {
         let bbox = if let Some(bbox) = self.get_bounds() {
             bbox
         } else {
-            [Vec2DDrawing::new(0.0, 0.0), Vec2DDrawing::new(0.0, 0.0)]
+            [Vec2DWorld::new(0.0, 0.0), Vec2DWorld::new(0.0, 0.0)]
         };
 
         self.storage.draw_commands(bbox)
     }
 
-    pub fn draw_commands_for_current_shape(&self) -> Option<DrawCommand> {
-        self.storage.draw_commands_for_current_shape()
-    }
-
-    /// Move the canvas by this offset
-    pub fn translate(&mut self, delta: Vec2DScreen) {
-        self.transform.translate_screen(delta);
+    /// returns only the draw commands needed to paint the current shape on the
+    /// screen.
+    pub fn draw_commands_for_current_shape(&self) -> Option<Vec<DrawCommand>> {
+        self.current_shape.as_ref().map(|s| s.draw_commands())
     }
 
     /// Notify the application that the screen size has changed
     pub fn resize(&mut self, new_size: Vec2DScreen) {
+        log::debug!("resize({})", new_size);
+
         let delta = (self.dimensions * -1.0 + new_size) * 0.5;
 
         self.translate(delta.into());
@@ -369,31 +411,43 @@ impl App {
 
     /// Set a new tool used to handle user input
     pub fn set_tool(&mut self, selected_tool: SelectedTool) {
+        log::debug!("set_tool({:?})", selected_tool);
+
         self.selected_tool = selected_tool;
     }
 
     /// Set a new stroke that will be used in next shapes
     pub fn set_stroke(&mut self, stroke: f64) {
+        log::debug!("set_stroke({})", stroke);
+
         self.stroke = stroke;
     }
 
     pub fn set_alpha(&mut self, alpha: f64) {
+        log::debug!("set_alpha({})", alpha);
+
         self.alpha = alpha;
     }
 
     /// Set the background color of the drawing
     pub fn set_bgcolor(&mut self, bgcolor: Color) {
+        log::debug!("set_bgcolor({})", bgcolor);
+
         self.bgcolor = bgcolor;
     }
 
     /// Set the radius of action for the eraser tool
     pub fn set_erase_radius(&mut self, erase_radius: f64) {
+        log::debug!("set_erase_radius({})", erase_radius);
+
         self.erase_radius = erase_radius;
     }
 
-    /// Public method that the UI must call whenever a button of the mouse or pen
-    /// is pressed.
+    /// Public method that the UI must call whenever a button of the mouse or
+    /// pen is pressed.
     pub fn handle_mouse_button_pressed(&mut self, button: MouseButton, point: Vec2DScreen) -> ShouldRedraw {
+        log::debug!("handle_mouse_button_pressed({:?}, {:?})", button, point);
+
         let (new_board_state, should_redraw) = match self.board_state {
             current@BoardState::Idle => match button {
                 MouseButton::Left => (BoardState::UsingTool, self.handle_tool_button_pressed(point)),
@@ -419,6 +473,8 @@ impl App {
     /// Public method that the UI must call whenever a button of the mouse or pen
     /// is released.
     pub fn handle_mouse_button_released(&mut self, button: MouseButton, point: Vec2DScreen) -> ShouldRedraw {
+        log::debug!("handle_mouse_button_released({:?}, {:?})", button, point);
+
         let (new_board_state, should_redraw) = match self.board_state {
             current@BoardState::Idle => (current, ShouldRedraw::No),
             current@BoardState::UsingTool => match button {
@@ -437,7 +493,7 @@ impl App {
             BoardState::Moving(old_point) => match button {
                 MouseButton::Left => (BoardState::Moving(old_point), ShouldRedraw::No),
                 MouseButton::Middle => {
-                    self.translate(point - old_point);
+                    self.handle_move(old_point, point);
 
                     (BoardState::Idle, ShouldRedraw::All)
                 },
@@ -456,7 +512,7 @@ impl App {
                         }
                 },
                 MouseButton::Middle => {
-                    self.translate(point - old_point);
+                    self.handle_move(old_point, point);
 
                     (BoardState::UsingTool, ShouldRedraw::All)
                 },
@@ -472,16 +528,18 @@ impl App {
 
     /// Public methid that the UI must call with updates on the cursor position
     pub fn handle_mouse_move(&mut self, pos: Vec2DScreen) -> ShouldRedraw {
+        log::debug!("handle_mouse_move({:?})", pos);
+
         let (new_board_state, should_redraw) = match self.board_state {
             BoardState::Idle => (BoardState::Idle, ShouldRedraw::No),
             BoardState::UsingTool => (BoardState::UsingTool, self.handle_tool_mouse_move(pos)),
             BoardState::Moving(old_point) => {
-                self.translate(pos - old_point);
+                self.handle_move(old_point, pos);
 
                 (BoardState::Moving(pos), ShouldRedraw::All)
             },
             BoardState::MovingWhileUsingTool(old_point) => {
-                self.translate(pos - old_point);
+                self.handle_move(old_point, pos);
 
                 (BoardState::MovingWhileUsingTool(pos), ShouldRedraw::All)
             },
@@ -492,30 +550,82 @@ impl App {
         should_redraw
     }
 
+    pub fn handle_key_pressed(&mut self, key: Key) {
+        log::debug!("handle_key_pressed({:?})", key);
+
+        match key {
+            Key::Shift => { self.is_shift_pressed = true; }
+            _ => { }
+        }
+    }
+
+    pub fn handle_key_released(&mut self, key: Key) {
+        log::debug!("handle_key_released({:?})", key);
+
+        match key {
+            Key::Shift => { self.is_shift_pressed = false; }
+            _ => { }
+        }
+    }
+
     /// Zooms in the current view, so objects are bigger and the visible portion
     /// of the drawing gets reduced
     pub fn zoom_in(&mut self) {
-        self.transform.zoom_in(2.0, self.dimensions);
+        log::debug!("zoom_in()");
+        self.transform = self.transform.zoom(2.0, self.dimensions / 2.0);
     }
 
     /// Zooms out the current view, so objects are smaller and the visible portion
     /// of the drawing increases
     pub fn zoom_out(&mut self) {
-        self.transform.zoom_out(2.0, self.dimensions);
+        log::debug!("zoom_out()");
+        self.transform = self.transform.zoom(0.5, self.dimensions / 2.0);
+    }
+
+    pub fn scroll_up(&mut self) {
+        log::debug!("scroll_up()");
+        if self.is_shift_pressed {
+            self.rotate(Angle::from_degrees(10.0), self.dimensions / 2.0);
+        } else {
+            self.translate(Vec2DScreen::new(0.0, 10.0));
+        }
+    }
+
+    pub fn scroll_down(&mut self) {
+        log::debug!("scroll_down()");
+        if self.is_shift_pressed {
+            self.rotate(Angle::from_degrees(-10.0), self.dimensions / 2.0);
+        } else {
+            self.translate(Vec2DScreen::new(0.0, -10.0));
+        }
+    }
+
+    pub fn scroll_right(&mut self) {
+        log::debug!("scroll_right()");
+        self.translate(Vec2DScreen::new(-10.0, 0.0));
+    }
+
+    pub fn scroll_left(&mut self) {
+        log::debug!("scroll_left()");
+        self.translate(Vec2DScreen::new(10.0, 0.0));
     }
 
     /// Resets the center and zoom level of the view to its original place
     pub fn go_home(&mut self) {
-        self.transform.go_home(self.dimensions / 2.0);
+        log::debug!("go_home()");
+        self.transform = Transform::default_for_viewport(self.dimensions);
     }
 
     /// Sets the color for the new shapes
     pub fn set_color(&mut self, color: Color) {
+        log::debug!("set_color({})", color);
         self.selected_color = color;
     }
 
     /// Undoes the last action if any
     pub fn undo(&mut self) {
+        log::debug!("undo()");
+
         match self.undo_state {
             // this is the first undo of this queue, we'll invert the last state
             // to its oposite and move the pointer backwards
@@ -552,6 +662,8 @@ impl App {
 
     /// Redoes the last undone action, if any
     pub fn redo(&mut self) {
+        log::debug!("redo()");
+
         match self.undo_state {
             // nothing has to be done for nothing has been undone
             UndoState::InSync => {},
@@ -585,13 +697,8 @@ impl App {
         }
     }
 
-    /// Return the drawing's total number of shapes
-    pub fn shape_count(&self) -> usize {
-        self.storage.shape_count()
-    }
-
     /// Returns the bounding box that contains all the shapes in the drawing
-    pub fn get_bounds(&self) -> Option<[Vec2DDrawing; 2]> {
+    pub fn get_bounds(&self) -> Option<[Vec2DWorld; 2]> {
         self.storage.get_bounds()
     }
 
@@ -608,9 +715,11 @@ impl App {
     /// Resets the state of the drawing, undo, shapes etc so you can start
     /// drawing again.
     pub fn reset(&mut self) {
+        log::debug!("reset()");
+        // TODO reconsider this, I suspect its implementation tends to be broken
         self.storage = Storage::new();
         self.selected_tool = SelectedTool::Shape(ShapeType::Path);
-        self.transform = Transform::new(self.dimensions.to_vec2d() * -0.5, 1.0);
+        self.transform = Transform::default_for_viewport(self.dimensions);
         self.undo_state = UndoState::Reset;
         self.undo_queue = Vec::new();
         self.save_status = SaveStatus::NewAndEmpty;
@@ -635,8 +744,8 @@ mod tests {
     use PathCommand::*;
 
     #[test]
-    fn test_draw_a_line() {
-        let mut app = App::new(Vec2DScreen::new(80.0, 60.0));
+    fn draw_a_line() {
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Path));
 
@@ -644,18 +753,18 @@ mod tests {
         app.handle_mouse_move(Vec2DScreen::new(1.0, 0.0));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(2.0, 0.0));
 
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.storage.shape_count(), 1);
 
         let commands = app.draw_commands_for_screen();
 
         assert_eq!(commands[0], DrawCommand::Path {
             color: Default::default(),
             commands: vec![
-                MoveTo(Vec2DDrawing { x: -40.0, y: -30.0 }),
+                MoveTo(Vec2DWorld { x: -40.0, y: -30.0 }),
                 CurveTo(CubicBezierCurve {
-                    pt1: Vec2DDrawing { x: -39.666666666666664, y: -30.0 },
-                    pt2: Vec2DDrawing { x: -39.333333333333336, y: -30.0 },
-                    to: Vec2DDrawing { x: -39.0, y: -30.0 }
+                    pt1: Vec2DWorld { x: -39.666666666666664, y: -30.0 },
+                    pt2: Vec2DWorld { x: -39.333333333333336, y: -30.0 },
+                    to: Vec2DWorld { x: -39.0, y: -30.0 }
                 }),
             ],
             thickness: DEFAULT_THICKNESS,
@@ -663,8 +772,8 @@ mod tests {
     }
 
     #[test]
-    fn test_draw_a_line_in_zoom() {
-        let mut app = App::new(Vec2DScreen::new(80.0, 60.0));
+    fn draw_a_line_in_zoom() {
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Path));
 
@@ -674,18 +783,18 @@ mod tests {
         app.handle_mouse_move(Vec2DScreen::new(1.0, 0.0));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(2.0, 0.0));
 
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.storage.shape_count(), 1);
 
         let commands = app.draw_commands_for_screen();
 
         assert_eq!(commands[0], DrawCommand::Path {
             color: Default::default(),
             commands: vec![
-                MoveTo(Vec2DDrawing { x: -20.0, y: -15.0 }),
+                MoveTo(Vec2DWorld { x: -20.0, y: -15.0 }),
                 CurveTo(CubicBezierCurve {
-                    pt1: Vec2DDrawing { x: -19.833333333333332, y: -15.0 },
-                    pt2: Vec2DDrawing { x: -19.666666666666668, y: -15.0 },
-                    to: Vec2DDrawing { x: -19.5, y: -15.0 }
+                    pt1: Vec2DWorld { x: -19.833333333333332, y: -15.0 },
+                    pt2: Vec2DWorld { x: -19.666666666666668, y: -15.0 },
+                    to: Vec2DWorld { x: -19.5, y: -15.0 }
                 }),
             ],
             thickness: DEFAULT_THICKNESS/2.0,
@@ -693,8 +802,8 @@ mod tests {
     }
 
     #[test]
-    fn test_erase() {
-        let mut app = App::new(Vec2DScreen::new(80.0, 60.0));
+    fn erase() {
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Path));
 
@@ -703,7 +812,7 @@ mod tests {
         app.handle_mouse_move(Vec2DScreen::new(15.0, 15.0));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(10.0, 15.0));
 
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.storage.shape_count(), 1);
 
         app.set_tool(SelectedTool::Eraser);
 
@@ -711,12 +820,12 @@ mod tests {
         app.handle_mouse_move(Vec2DScreen::new(15.0, 10.0));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(15.0, 10.0));
 
-        assert_eq!(app.shape_count(), 0);
+        assert_eq!(app.storage.shape_count(), 0);
     }
 
     #[test]
-    fn test_erase_with_zoom() {
-        let mut app = App::new(Vec2DScreen::new(4.0, 4.0));
+    fn erase_with_zoom() {
+        let mut app = Pizarra::new(Vec2DScreen::new(4.0, 4.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Path));
 
@@ -726,7 +835,7 @@ mod tests {
         app.handle_mouse_move(Vec2DScreen::new(2.0, 2.5));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(2.0, 3.0));
 
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.storage.shape_count(), 1);
 
         app.zoom_in();
 
@@ -738,19 +847,28 @@ mod tests {
         app.handle_mouse_move(Vec2DScreen::new(2.6, 3.0));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(2.6, 4.0));
 
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.storage.shape_count(), 1);
 
         app.handle_mouse_button_pressed(MouseButton::Left, Vec2DScreen::new(2.24, 0.0));
         app.handle_mouse_move(Vec2DScreen::new(2.4, 1.0));
         app.handle_mouse_move(Vec2DScreen::new(2.4, 3.0));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(2.24, 4.0));
 
-        assert_eq!(app.shape_count(), 0);
+        assert_eq!(app.storage.shape_count(), 0);
     }
 
     #[test]
-    fn test_do_do_do_undo_redo_redo_undo() {
-        let mut app = App::new(Vec2DScreen::new(800.0, 600.0));
+    fn zoom_out_is_plugged() {
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
+
+        app.zoom_out();
+
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-80.0, -60.0), Vec2DWorld::new(80.0, 60.0)]);
+    }
+
+    #[test]
+    fn do_do_do_undo_redo_redo_undo() {
+        let mut app = Pizarra::new(Vec2DScreen::new(800.0, 600.0));
 
         // Do
         app.handle_mouse_button_pressed(MouseButton::Left, Vec2DScreen::new(0.0, 1.0));
@@ -774,12 +892,12 @@ mod tests {
 
         app.undo();
 
-        assert_eq!(app.shape_count(), 2);
+        assert_eq!(app.storage.shape_count(), 2);
     }
 
     #[test]
-    fn test_do_undo_redo_undo() {
-        let mut app = App::new(Vec2DScreen::new(800.0, 600.0));
+    fn do_undo_redo_undo() {
+        let mut app = Pizarra::new(Vec2DScreen::new(800.0, 600.0));
 
         // Do
         app.handle_mouse_button_pressed(MouseButton::Left, Vec2DScreen::new(0.0, 1.0));
@@ -792,12 +910,12 @@ mod tests {
 
         app.undo();
 
-        assert_eq!(app.shape_count(), 0);
+        assert_eq!(app.storage.shape_count(), 0);
     }
 
     #[test]
-    fn test_do_undo_do_do_undo() {
-        let mut app = App::new(Vec2DScreen::new(800.0, 600.0));
+    fn do_undo_do_do_undo() {
+        let mut app = Pizarra::new(Vec2DScreen::new(800.0, 600.0));
 
         // Do
         app.handle_mouse_button_pressed(MouseButton::Left, Vec2DScreen::new(0.0, 1.0));
@@ -818,12 +936,12 @@ mod tests {
 
         app.undo();
 
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.storage.shape_count(), 1);
     }
 
     #[test]
-    fn test_do_undo_do_undo_undo() {
-        let mut app = App::new(Vec2DScreen::new(800.0, 600.0));
+    fn do_undo_do_undo_undo() {
+        let mut app = Pizarra::new(Vec2DScreen::new(800.0, 600.0));
 
         // Do
         app.handle_mouse_button_pressed(MouseButton::Left, Vec2DScreen::new(0.0, 1.0));
@@ -840,42 +958,42 @@ mod tests {
         app.undo();
         app.undo();
 
-        assert_eq!(app.shape_count(), 0);
+        assert_eq!(app.storage.shape_count(), 0);
     }
 
     #[test]
-    fn test_move() {
-        let mut app = App::new(Vec2DScreen::new(80.0, 60.0));
+    fn r#move() {
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-40.0, -30.0), Vec2DDrawing::new(40.0, 30.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-40.0, -30.0), Vec2DWorld::new(40.0, 30.0)]);
 
         app.handle_mouse_button_pressed(MouseButton::Middle, Vec2DScreen::new(0.0, 0.0));
         app.handle_mouse_button_released(MouseButton::Middle, Vec2DScreen::new(-1.0, 0.0));
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-39.0, -30.0), Vec2DDrawing::new(41.0, 30.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-39.0, -30.0), Vec2DWorld::new(41.0, 30.0)]);
     }
 
     #[test]
     fn move_while_in_zoom_is_coherent() {
-        let mut app = App::new(Vec2DScreen::new(80.0, 60.0));
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-40.0, -30.0), Vec2DDrawing::new(40.0, 30.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-40.0, -30.0), Vec2DWorld::new(40.0, 30.0)]);
 
         app.zoom_in();
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-20.0, -15.0), Vec2DDrawing::new(20.0, 15.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-20.0, -15.0), Vec2DWorld::new(20.0, 15.0)]);
 
         app.handle_mouse_button_pressed(MouseButton::Middle, Vec2DScreen::new(0.0, 0.0));
         app.handle_mouse_button_released(MouseButton::Middle, Vec2DScreen::new(-1.0, 0.0));
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-19.5, -15.0), Vec2DDrawing::new(20.5, 15.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-19.5, -15.0), Vec2DWorld::new(20.5, 15.0)]);
     }
 
     #[test]
-    fn test_move_while_drawing() {
-        let mut app = App::new(Vec2DScreen::new(80.0, 60.0));
+    fn move_while_drawing() {
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-40.0, -30.0), Vec2DDrawing::new(40.0, 30.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-40.0, -30.0), Vec2DWorld::new(40.0, 30.0)]);
 
         app.set_tool(SelectedTool::Shape(ShapeType::Path));
 
@@ -887,8 +1005,8 @@ mod tests {
         app.handle_mouse_move(Vec2DScreen::new(40.0, 10.0));
         app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(40.0, 10.0));
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-50.0, -30.0), Vec2DDrawing::new(30.0, 30.0)]);
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-50.0, -30.0), Vec2DWorld::new(30.0, 30.0)]);
+        assert_eq!(app.storage.shape_count(), 1);
 
         let commands = app.draw_commands_for_screen();
 
@@ -898,39 +1016,39 @@ mod tests {
             color: Default::default(),
             thickness: DEFAULT_THICKNESS,
             commands: vec![
-                MoveTo(Vec2DDrawing { x: -30.0, y: -20.0 }),
+                MoveTo(Vec2DWorld { x: -30.0, y: -20.0 }),
                 CurveTo(CubicBezierCurve {
-                    pt1: Vec2DDrawing { x: -26.666666666666668, y: -20.0 },
-                    pt2: Vec2DDrawing { x: -23.333333333333332, y: -20.0 },
-                    to: Vec2DDrawing { x: -20.0, y: -20.0 }
+                    pt1: Vec2DWorld { x: -26.666666666666668, y: -20.0 },
+                    pt2: Vec2DWorld { x: -23.333333333333332, y: -20.0 },
+                    to: Vec2DWorld { x: -20.0, y: -20.0 }
                 }),
                 CurveTo(CubicBezierCurve {
-                    pt1: Vec2DDrawing { x: -16.666666666666668, y: -20.0 },
-                    pt2: Vec2DDrawing { x: -13.333333333333332, y: -20.0 },
-                    to: Vec2DDrawing { x: -10.0, y: -20.0 }
+                    pt1: Vec2DWorld { x: -16.666666666666668, y: -20.0 },
+                    pt2: Vec2DWorld { x: -13.333333333333332, y: -20.0 },
+                    to: Vec2DWorld { x: -10.0, y: -20.0 }
                 }),
             ],
         });
     }
 
     #[test]
-    fn test_zoom_must_be_idempotent() {
-        let mut app = App::new(Vec2DScreen::new(80.0, 60.0));
+    fn zoom_must_be_idempotent() {
+        let mut app = Pizarra::new(Vec2DScreen::new(80.0, 60.0));
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-40.0, -30.0), Vec2DDrawing::new(40.0, 30.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-40.0, -30.0), Vec2DWorld::new(40.0, 30.0)]);
 
         app.zoom_in();
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-20.0, -15.0), Vec2DDrawing::new(20.0, 15.0)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-20.0, -15.0), Vec2DWorld::new(20.0, 15.0)]);
 
         app.zoom_in();
 
-        assert_eq!(app.visible_extent(), [Vec2DDrawing::new(-10.0, -7.5), Vec2DDrawing::new(10.0, 7.5)]);
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-10.0, -7.5), Vec2DWorld::new(10.0, 7.5)]);
     }
 
     #[test]
-    fn test_cache_invalidation_on_ctrl_z() {
-        let mut app = App::new(Vec2DScreen::new(10.0, 10.0));
+    fn cache_invalidation_on_ctrl_z() {
+        let mut app = Pizarra::new(Vec2DScreen::new(10.0, 10.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Rectangle));
 
@@ -975,7 +1093,7 @@ mod tests {
     }
 
     #[test]
-    fn test_not_erasing_a_shape_doesnt_leave_the_eraser_active() {
+    fn not_erasing_a_shape_doesnt_leave_the_eraser_active() {
         // the scenario is: you draw a few shapes, then change to the eraser, click and drag
         // on the screen but without touching a shape and then release the left button.
         //
@@ -983,7 +1101,7 @@ mod tests {
         // it gets erased
         //
         // it's expeceted for the eraser not to erase the shape
-        let mut app = App::new(Vec2DScreen::new(100.0, 100.0));
+        let mut app = Pizarra::new(Vec2DScreen::new(100.0, 100.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Rectangle));
 
@@ -1001,6 +1119,46 @@ mod tests {
         // move to where the shape is
         app.handle_mouse_move(Vec2DScreen::new(5.0, 5.0));
 
-        assert_eq!(app.shape_count(), 1);
+        assert_eq!(app.storage.shape_count(), 1);
+    }
+
+    #[test]
+    fn app_state_after_finished_shape() {
+        let mut app = Pizarra::new(Vec2DScreen::new(100.0, 100.0));
+
+        app.set_tool(SelectedTool::Shape(ShapeType::Path));
+
+        let p1 = Vec2DScreen::new(1.0, 1.0);
+        let p2 = Vec2DScreen::new(20.0, 20.0);
+
+        app.handle_mouse_button_pressed(MouseButton::Left, p1);
+        app.handle_mouse_move(p2);
+        app.handle_mouse_button_released(MouseButton::Left, p2);
+
+        assert_eq!(app.board_state, BoardState::Idle);
+        assert_eq!(app.undo_state, UndoState::InSync);
+        assert_eq!(app.save_status, SaveStatus::NewAndChanged);
+        assert_eq!(app.storage.shape_count(), 1);
+
+        if let Some(_) = app.current_shape.as_ref() {
+            panic!()
+        }
+    }
+
+    #[test]
+    fn resize_screen_keeps_center() {
+        let mut app = Pizarra::new(Vec2DScreen::new(100.0, 100.0));
+
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-50.0, -50.0), Vec2DWorld::new(50.0, 50.0)]);
+
+        app.resize(Vec2DScreen::new(200.0, 300.0));
+
+        assert_eq!(app.visible_bbox(), [Vec2DWorld::new(-100.0, -150.0), Vec2DWorld::new(100.0, 150.0)]);
+    }
+
+    #[test]
+    #[ignore]
+    fn esc_clears_current_shape() {
+        assert!(false, "pressing the esc button should clear the current shape");
     }
 }
