@@ -1,9 +1,9 @@
 use xml::writer::{EmitterConfig, XmlEvent, EventWriter};
 
 use crate::consts::EXPORT_PADDING;
-use crate::point::Vec2DDrawing;
+use crate::point::{Point, Vec2DWorld};
 use crate::draw_commands::DrawCommand;
-use crate::app::App;
+use crate::app::Pizarra;
 
 impl DrawCommand {
     fn serialize(self, writer: &mut EventWriter<&mut Vec<u8>>) {
@@ -41,22 +41,25 @@ impl DrawCommand {
                 writer.write(XmlEvent::end_element()).unwrap();
             },
             DrawCommand::Ellipse {
-                bbox, thickness, color,
+                thickness, color, center, semimajor, semiminor, angle,
             } => {
-                let min = bbox[0].min(bbox[1]);
-                let max = bbox[0].max(bbox[1]);
-
-                let bounds = max - min;
-                let half = bounds/2.0;
-                let center = half + min;
-
                 writer.write(XmlEvent::start_element("ellipse")
                     .attr("cx", &center.x.to_string())
                     .attr("cy", &center.y.to_string())
-                    .attr("rx", &half.x.to_string())
-                    .attr("ry", &half.y.to_string())
-                    .attr("style", &format!(
-                        "fill:none;stroke-width:{stroke};stroke:{color};stroke-opacity:{alpha};stroke-miterlimit:10",
+                    .attr("rx", &semimajor.to_string())
+                    .attr("ry", &semiminor.to_string())
+                    .attr("transform", &format!(
+                            "rotate({angle} {x} {y})",
+                            angle = angle.degrees(),
+                            x = center.x,
+                            y = center.y,
+                    ))
+                    .attr("style", &format!("\
+                            fill:none;\
+                            stroke-width:{stroke};\
+                            stroke:{color};\
+                            stroke-opacity:{alpha};\
+                            stroke-miterlimit:10;",
                         stroke = thickness,
                         color = color.css(),
                         alpha = color.alpha(),
@@ -68,7 +71,7 @@ impl DrawCommand {
     }
 }
 
-impl App {
+impl Pizarra {
     pub fn to_svg(&self) -> Option<String> {
         if let Some(bbox) = self.get_bounds() {
             let mut output = Vec::new();
@@ -76,7 +79,7 @@ impl App {
                 .perform_indent(true)
                 .create_writer(&mut output);
 
-            let svg_dimensions = (bbox[0] - bbox[1]).abs() + Vec2DDrawing::new(EXPORT_PADDING*2.0, EXPORT_PADDING*2.0);
+            let svg_dimensions = (bbox[0] - bbox[1]).abs() + Vec2DWorld::new(EXPORT_PADDING*2.0, EXPORT_PADDING*2.0);
             let width = svg_dimensions.x.to_string();
             let height = svg_dimensions.y.to_string();
             let min = bbox[0].min(bbox[1]);
@@ -110,6 +113,8 @@ impl App {
             writer.write(XmlEvent::end_element()).unwrap(); // g
             writer.write(XmlEvent::end_element()).unwrap(); // svg
 
+            output.push(b'\n');
+
             Some(String::from_utf8(output).unwrap())
         } else {
             None
@@ -119,14 +124,16 @@ impl App {
 
 #[cfg(test)]
 mod tests {
+    use super::*;
+
     use crate::color::Color;
     use crate::point::Vec2DScreen;
-    use crate::app::{App, MouseButton, SelectedTool};
+    use crate::app::{Pizarra, MouseButton, SelectedTool};
     use crate::shape::ShapeType;
 
     #[test]
     fn test_serialize() {
-        let mut app = App::new(Vec2DScreen::new(40.0, 40.0));
+        let mut app = Pizarra::new(Vec2DScreen::new(40.0, 40.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Path));
         app.set_color(Color::red());
@@ -140,21 +147,34 @@ mod tests {
 
     #[test]
     fn serialize_ellipse() {
-        let mut app = App::new(Vec2DScreen::new(40.0, 40.0));
+        let mut app = Pizarra::new(Vec2DScreen::new(40.0, 40.0));
 
-        app.set_tool(SelectedTool::Shape(ShapeType::Ellipse));
+        app.set_tool(SelectedTool::Shape(ShapeType::ThreePointEllipse));
         app.set_color(Color::red());
         app.set_stroke(3.5);
 
-        app.handle_mouse_button_pressed(MouseButton::Left, Vec2DScreen::new(5.0, 10.0));
-        app.handle_mouse_button_released(MouseButton::Left, Vec2DScreen::new(35.0, 30.0));
+        let a: Vec2DScreen = (-5.61146,3.80508).into();
+        let c: Vec2DScreen = (-0.44,-1.28).into();
+        let e: Vec2DScreen = (4.3,-0.4).into();
+
+        // start and first focus of the ellipse
+        app.handle_mouse_button_pressed(MouseButton::Left, a);
+        app.handle_mouse_button_released(MouseButton::Left, a);
+
+        // second focus of the ellipse
+        app.handle_mouse_move(c);
+        app.handle_mouse_button_released(MouseButton::Left, c);
+
+        // external point of the ellipse, finish shape
+        app.handle_mouse_move(e);
+        app.handle_mouse_button_released(MouseButton::Left, e);
 
         assert_eq!(app.to_svg().unwrap(), include_str!("../res/serialize_ellipse.svg"));
     }
 
     #[test]
     fn test_serialize_color_with_alpha() {
-        let mut app = App::new(Vec2DScreen::new(40.0, 40.0));
+        let mut app = Pizarra::new(Vec2DScreen::new(40.0, 40.0));
 
         app.set_tool(SelectedTool::Shape(ShapeType::Polygon));
         app.set_color(Color::green());
